package com.Shawn.service;

import com.Shawn.feign.UserFeign;
import com.alibaba.csp.sentinel.annotation.SentinelResource;
import com.alibaba.csp.sentinel.slots.block.BlockException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.shaded.io.grpc.netty.shaded.io.netty.handler.codec.http.HttpUtil;
import com.shawn.common.Constants.RedisConstants;
import com.shawn.common.Exception.LoginCodeInvalidException;
import com.shawn.common.domain.Result;
import com.shawn.common.domain.User;
import com.shawn.common.utils.http.HttpUtils;
import com.shawn.common.utils.uuid.IdUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author Shawn
 */
@Service
public class LoginService {

    @Autowired
    StringRedisTemplate redisTemplate;

    @Autowired
    UserFeign userFeign;

    @SentinelResource(value = "authorize",blockHandler = "blockHandlerForHello")
    public Result authorize(String code) throws Exception{
        String rspStr = HttpUtils.sendGet("https://api.weixin.qq.com/sns/jscode2session","appid=wx18ddfb7e334b966b&secret=a9fc333198df019622eb1f7a6e03390b&js_code="+code+"&grant_type=authorization_code", "UTF-8");
        if(rspStr.contains("errmsg")){
            throw new LoginCodeInvalidException(JSONObject.parseObject(rspStr, Map.class).get("errmsg").toString());
        }
        String openid = JSONObject.parseObject(rspStr, Map.class).get("openid").toString();
        String token = IdUtils.fastUUID();
        redisTemplate.opsForHash().put(RedisConstants.REDIS_USER_KEY+token,"user","null");
        redisTemplate.expire(RedisConstants.REDIS_USER_KEY+token,1,TimeUnit.HOURS);
        Map<String, String> res = new HashMap<>();
        res.put("token",token);
        res.put("openid",openid);
        return Result.success(res);
    }

    public String blockHandlerForHello(String name, BlockException ex) {
        return "当前访问人数过多，请稍候.....";
    }

    public Result checkToken(String token) {
        if (redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token,"user") == "null" || redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token,"user") == null){
            return Result.error();
        }else {
            return Result.success();
        }
    }

//    @PostConstruct
//    private static void initFlowRules(){
//        List<FlowRule> rules = new ArrayList<>();
//
//        FlowRule rule = new FlowRule();
//        rule.setResource("sayHello");
//        rule.setGrade(RuleConstant.FLOW_GRADE_QPS);
//
//        rule.setCount(200);
//        rules.add(rule);
//        FlowRuleManager.loadRules(rules);
//
//        List<DegradeRule> degradeRules = new ArrayList<>();
//        DegradeRule degradeRule = new DegradeRule();
//        degradeRule.setResource("sayHello");
//        degradeRule.setGrade(RuleConstant.DEGRADE_GRADE_EXCEPTION_COUNT);
//        degradeRule.setCount(3);
//        degradeRule.setStatIntervalMs(60*1000);
//        degradeRule.setTimeWindow(10);
//        degradeRules.add(degradeRule);
//        DegradeRuleManager.loadRules(degradeRules);
//
//    }

}
