package com.shawn.service.Impl;

import com.alibaba.fastjson.JSONObject;
import com.shawn.common.Constants.RedisConstants;
import com.shawn.common.Exception.TokenIsNullException;
import com.shawn.common.domain.Banner;
import com.shawn.common.domain.Result;
import com.shawn.common.domain.Teacher;
import com.shawn.common.utils.DistanceUtil;
import com.shawn.feign.UserFeign;
import com.shawn.mapper.PageMapper;
import com.shawn.service.IndexPageService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.cache.CacheProperties;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author Shawn
 */
@Service
public class IndexPageServiceImpl implements IndexPageService {

    @Autowired
    PageMapper pageMapper;

    @Autowired
    StringRedisTemplate redisTemplate;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @Autowired
    UserFeign userFeign;

    @Override
    public Result getBannerAndLBT(String type) {
        if(!redisTemplate.opsForHash().hasKey(RedisConstants.REDIS_PAGE_KEY,"IndexBanner") || !redisTemplate.opsForHash().hasKey(RedisConstants.REDIS_PAGE_KEY,"AppBanner")){
            if("app".equals(type)){
                List<Banner> bannerAndLBT = pageMapper.getBannerAndLBT(type);
                redisTemplate.opsForHash().put(RedisConstants.REDIS_PAGE_KEY,"AppBanner", JSONObject.toJSONString(bannerAndLBT));
                redisTemplate.expire(RedisConstants.REDIS_PAGE_KEY,30, TimeUnit.DAYS);
                return Result.success(bannerAndLBT);
            }else if("index".equals(type)){
                List<Banner> bannerAndLBT = pageMapper.getBannerAndLBT(type);
                redisTemplate.opsForHash().put(RedisConstants.REDIS_PAGE_KEY,"IndexBanner",JSONObject.toJSONString(bannerAndLBT));
                redisTemplate.expire(RedisConstants.REDIS_PAGE_KEY,30, TimeUnit.DAYS);
                return Result.success(bannerAndLBT);
            }
        }else{
            if ("app".equals(type)){
                List<Banner> appBanner = JSONObject.parseObject(String.valueOf(redisTemplate.opsForHash().get(RedisConstants.REDIS_PAGE_KEY, "AppBanner")), List.class);
                return Result.success(appBanner);
            }else{
                List<Banner> appBanner = JSONObject.parseObject(String.valueOf(redisTemplate.opsForHash().get(RedisConstants.REDIS_PAGE_KEY, "IndexBanner")), List.class);
                return Result.success(appBanner);
            }
        }
        return Result.error();
    }

    @Override
    public Result getNearTeachers(String ip,String lat,String lng,String token) throws Exception {
        if(redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token,"user") == null){
            throw new TokenIsNullException("Token不存在，请重新进入小程序");
        }else{
            if(redisTemplate.hasKey(RedisConstants.REDIS_TEACHERONE_KEY+ip)){
                List<Teacher> nearTeachersListByIp = new ArrayList<>(6);
                //可能会有列表为空的情况,要在统一处理捕获
                Long size = redisTemplate.opsForZSet().size(RedisConstants.REDIS_TEACHERONE_KEY + ip);
                for (String teacher : Objects.requireNonNull(redisTemplate.opsForZSet().range(RedisConstants.REDIS_TEACHERONE_KEY + ip, 0, size>=5?5:size-1))) {
                    nearTeachersListByIp.add(JSONObject.parseObject(teacher,Teacher.class));
                }
                return Result.success(nearTeachersListByIp);
            }else if(redisTemplate.hasKey(RedisConstants.REDIS_TEACHERLIST_KEY)){
                Set<Object> uidAndLatAndLng = redisTemplate.opsForHash().keys(RedisConstants.REDIS_TEACHERLIST_KEY);
                for (Object o : uidAndLatAndLng) {
                    Teacher teacher = JSONObject.parseObject((String) redisTemplate.opsForHash().get(RedisConstants.REDIS_TEACHERLIST_KEY, o), Teacher.class);
                    assert teacher != null;
                    double distance = DistanceUtil.getDistance(Double.parseDouble(lng), Double.parseDouble(lat), teacher.getLng(), teacher.getLat());
                    teacher.setDistance(distance);
                    redisTemplate.opsForZSet().add(RedisConstants.REDIS_TEACHERONE_KEY + ip,JSONObject.toJSONString(teacher),distance);
                    redisTemplate.expire(RedisConstants.REDIS_TEACHERONE_KEY + ip,1,TimeUnit.HOURS);
                }
                Long size = redisTemplate.opsForZSet().size(RedisConstants.REDIS_TEACHERONE_KEY + ip);
                List<Teacher> nearTeachersListByIp = new ArrayList<>(6);
                //可能会有列表为空的情况,要在统一处理捕获
                for (String teacher : Objects.requireNonNull(redisTemplate.opsForZSet().range(RedisConstants.REDIS_TEACHERONE_KEY + ip, 0, size>=5?5:size-1))) {
                    nearTeachersListByIp.add(JSONObject.parseObject(teacher,Teacher.class));
                }
                return Result.success(nearTeachersListByIp);
            }else if(!redisTemplate.hasKey(RedisConstants.REDIS_TEACHERONE_KEY+ip) && !redisTemplate.hasKey(RedisConstants.REDIS_TEACHERLIST_KEY)){
                rabbitTemplate.convertAndSend("AddTeacherListQueue",RedisConstants.REDIS_TEACHERLIST_KEY);
                return userFeign.getNearTeacherList(ip, lat, lng);
            }
        }
        return null;
    }

    @Override
    public Result refreshNearTeachers(String ip, String lat, String lng, String token) {
        redisTemplate.expire(RedisConstants.REDIS_TEACHERONE_KEY+ip,1,TimeUnit.NANOSECONDS);
        return Result.success();
    }
}
