package com.shawn.user.listener;

import com.alibaba.fastjson.JSONObject;
import com.shawn.common.domain.Teacher;
import com.shawn.common.domain.User;
import com.shawn.user.Mapper.UserMapper;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author Shawn
 */
@Component
public class UserMQListener {

    @Autowired
    UserMapper userMapper;

    @Autowired
    StringRedisTemplate redisTemplate;

    @RabbitListener(queues = "UserInfoAddQueue")
    public void listenUserInfoAddQueueMessage(User user) throws InterruptedException {
        userMapper.insertUserInfo(user);
    }

    @RabbitListener(queues = "UserInfoModifyQueue")
    public void listenUserInfoModifyQueueMessage(User user) throws InterruptedException {
        userMapper.modifyUserInfo(user);
    }

    @RabbitListener(queues = "AddTeacherListQueue")
    public void listenAddTeacherListQueueMessage(String key) throws InterruptedException {
        if (!redisTemplate.hasKey(key)){
            synchronized (this){
                if (!redisTemplate.hasKey(key)) {
                    List<Teacher> verifiedTeacherList = userMapper.getVerifiedTeacherList();
                    Map<String, String> teacherMap = new HashMap<>();
                    verifiedTeacherList.forEach(teacher -> teacherMap.put(teacher.getUid() + "&" + teacher.getLat() + "&" + teacher.getLng(), JSONObject.toJSONString(teacher)));
                    redisTemplate.opsForHash().putAll(key, teacherMap);
                    redisTemplate.expire(key,7, TimeUnit.DAYS);
                }
            }
        }
    }

}
