package com.shawn.user.service.Impl;

import com.alibaba.csp.sentinel.annotation.SentinelResource;
import com.alibaba.csp.sentinel.slots.block.BlockException;
import com.alibaba.csp.sentinel.slots.block.RuleConstant;
import com.alibaba.csp.sentinel.slots.block.flow.FlowRule;
import com.alibaba.csp.sentinel.slots.block.flow.FlowRuleManager;
import com.alibaba.fastjson.JSONObject;

import com.shawn.common.Constants.RedisConstants;
import com.shawn.common.Exception.IPOrLocationIsInvalidException;
import com.shawn.common.Exception.TokenIsNullException;
import com.shawn.common.domain.Result;
import com.shawn.common.domain.Teacher;
import com.shawn.common.domain.User;
import com.shawn.common.utils.DateUtils;
import com.shawn.common.utils.DistanceUtil;
import com.shawn.common.utils.StringUtils;
import com.shawn.user.Mapper.UserMapper;
import com.shawn.user.service.userService;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author Shawn
 */
@Service
public class UserServiceImpl implements userService {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    RestTemplate restTemplate;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @Resource
    private RedissonClient redissonClient;

    @Override
//    @SentinelResource(value = "getUserDetailByOpenId",blockHandler = "blockHandlerForGetUserDetailByOpenId")
    public Result getUserDetailByToken(String token) throws TokenIsNullException {
//        if (openid != null){
//            if (userMapper.getUserDetailByOpenId(openid) != null){
//                return Result.success(userMapper.getUserDetailByOpenId(openid));
//            }else {
//                return Result.error();
//            }
//        }else
        if(token != null){
            if (redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token, "user") == null){
                throw new TokenIsNullException("Token异常,请重新进入小程序");
            }
            if(StringUtils.equals((String)redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token, "user"), "null")){
                return Result.success(new User());
            }else{
                User user = JSONObject.parseObject((String) redisTemplate.opsForHash().get(RedisConstants.REDIS_USER_KEY+token, "user"), User.class);
                System.out.println(user.toString());
                return Result.success(user);
            }
        }else {
            throw new TokenIsNullException("Token不存在");
        }
    }

    @Override
    public Result loginAndModifyUserInfo(String token, String openid, String alias, String avatarUrl) {
        User userDetailByOpenId = userMapper.getUserDetailByOpenId(openid);
        //数据库中无用户数据（首次登陆）
        if (userDetailByOpenId == null){
            User user = new User();
            String leafUid = restTemplate.getForObject("http://127.0.0.1:9001/api/segment/get/shawn", String.class);
            user.setUid(DateUtils.dateTimeNow().substring(2,10)+leafUid);
            user.setAlias(alias);
            user.setFirst_login_datetime(DateUtils.getTime());
            user.setLast_login_datetime(DateUtils.getTime());
            user.setOpenid(openid);
            user.setAvatar(avatarUrl);
            redisTemplate.opsForHash().put(RedisConstants.REDIS_USER_KEY+token,"user",JSONObject.toJSONString(user));
            redisTemplate.expire(RedisConstants.REDIS_USER_KEY+token,7, TimeUnit.DAYS);
            rabbitTemplate.convertAndSend("UserInfoExchange","UserAdd", user);
            return Result.success();
        }else{
            userDetailByOpenId.setAlias(alias);
            userDetailByOpenId.setAvatar(avatarUrl);
            redisTemplate.opsForHash().put(RedisConstants.REDIS_USER_KEY+token,"user",JSONObject.toJSONString(userDetailByOpenId));
            redisTemplate.expire(RedisConstants.REDIS_USER_KEY+token,7, TimeUnit.DAYS);
            userDetailByOpenId.setLast_login_datetime(DateUtils.getTime());
            rabbitTemplate.convertAndSend("UserInfoExchange","UserModify", userDetailByOpenId);
            return Result.success();
        }
    }

    @Override
    @SentinelResource(value = "getNearTeacherList",blockHandler = "blockHandlerForgetNearTeacherListBlockException")
    public Result getNearTeacherList(String ip, String lat, String lng) throws Exception {
        if(StringUtils.isEmpty(ip) || StringUtils.isEmpty(ip) || StringUtils.isEmpty(ip)){
            throw new IPOrLocationIsInvalidException("位置信息获取失败，请您检查您的网络设置！");
        }
        //这里需要加一个分布式锁，防止同时有人进行插入redis
        RLock lock = redissonClient.getLock(RedisConstants.REDIS_LOCKS_KEY + ip);
        boolean isLock = lock.tryLock(1,4,TimeUnit.SECONDS);
        if (isLock){
            List<Teacher> verifiedTeacherList = userMapper.getVerifiedTeacherList();
            verifiedTeacherList.forEach(teacher -> {
                teacher.setDistance(DistanceUtil.getDistance(Double.parseDouble(lng),Double.parseDouble(lat),teacher.getLng(),teacher.getLat()));
            });
            verifiedTeacherList.sort(Comparator.comparingDouble(Teacher::getDistance));
            for (Teacher teacher : verifiedTeacherList) {
                redisTemplate.opsForZSet().add(RedisConstants.REDIS_TEACHERONE_KEY + ip,JSONObject.toJSONString(teacher),teacher.getDistance());
                redisTemplate.expire(RedisConstants.REDIS_TEACHERONE_KEY + ip,1,TimeUnit.HOURS);
            }
            lock.unlock();
            return Result.success(verifiedTeacherList);
        }else{
            return Result.error("当前网络不佳，请稍后重试！");
        }
    }

//    @Override
//    public Result getUserDetailByToken(String token) {
//        User user = JSONObject.parseObject((byte[]) Objects.requireNonNull(redisTemplate.opsForHash().get(token, "user")), User.class);
//        return Result.success(user);
//    }

    public Result blockHandlerForGetUserDetailByOpenId(String openid, BlockException ex) {
        return Result.error("当前访问人数过多，请稍候.....");
    }

    public Result blockHandlerForgetNearTeacherListBlockException(String ip, String lat, String lng, BlockException ex) {
        ex.printStackTrace();
        return Result.error("当前访问人数过多，请稍候");
    }

    @PostConstruct
    private static void initFlowRules(){
        List<FlowRule> rules = new ArrayList<>();

        FlowRule rule = new FlowRule();
        rule.setResource("getNearTeacherList");
        rule.setGrade(RuleConstant.FLOW_GRADE_QPS);

        rule.setCount(2000);
        rules.add(rule);
        FlowRuleManager.loadRules(rules);

//        List<DegradeRule> degradeRules = new ArrayList<>();
//        DegradeRule degradeRule = new DegradeRule();
//        degradeRule.setResource("sayHello");
//        degradeRule.setGrade(RuleConstant.DEGRADE_GRADE_EXCEPTION_COUNT);
//        degradeRule.setCount(3);
//        degradeRule.setStatIntervalMs(60*1000);
//        degradeRule.setTimeWindow(10);
//        degradeRules.add(degradeRule);
//        DegradeRuleManager.loadRules(degradeRules);

    }
}
